
Card = class('Card')

function Card:initialize(n)
	assert(type(n)=='string', "card initialized with a non-string name")
	self.name = n
	self.image = getCardImage(n)
	self.tapped = 0
	self.hover = false
	self.x = 0
	self.y = 0
	self.size = math.min(sw/(223*8), sh/(310*4))
	self.xoffset = self.image:getWidth()/2
	self.yoffset = self.image:getHeight()/2
end

function Card:update()
	if self:isOver(lm.getPosition()) then self.hover = true else self.hover = false end
end

function Card:draw()
	lg.draw(self.image,self.x,self.y,self.tapped*math.pi*(1/2),self.size,self.size,self.xoffset,self.yoffset)
end

function Card:isOver(mx,my)
	if self.tapped == 0 then
		if mx > (self.x-self.xoffset*self.size) and mx < (self.x+self.xoffset*self.size) and my > (self.y-self.yoffset*self.size) and my < (self.y+self.yoffset*self.size) then
			self.hover = true
			return true
		else
			self.hover = false
			return false
		end
	else
		if mx > (self.x-self.yoffset*self.size) and mx < (self.x+self.yoffset*self.size) and my > (self.y-self.xoffset*self.size) and my < (self.y+self.xoffset*self.size) then
			self.hover = true
			return true
		else
			self.hover = false
			return false
		end
	end
end

---------------------------------------------------

function getCardImage(card)
	if card == '' then return nil end
	local n2 = string.lower(card)
	local n = ''		--name to store card with, will be lower case and only contains characters
	for l in n2:gmatch('%l') do
		n = n .. l
	end
	if img[n .. 'card']==nil then
		assert(type(card)=='string' or type(card)=='number', 'tried to get an image of a card with a non-string label')
		local s = string.gsub(card, "%s", "%%20")
		local url = 'http://gatherer.wizards.com/Handlers/Image.ashx?name=' .. s .. '&type=card'
		local d = li.newImageData(lfs.newFileData(http.request(url), '', 'file'))
		if isNullCard(d) then
			return nil
		else
			clipCardCorners(d)
			d:encode(n .. 'card.png')
			local i = lg.newImage(d)
			img[n .. 'card'] = i
			return img[n .. 'card']
		end
	--we already had the card image
	else
		return img[n .. 'card']
	end
end

function clipCardCorners(d)	--d is card ImageData
	local h = d:getHeight()
	local w = d:getWidth()
	for i = 1,6 do
		for j = 1,6 do
			if i+j < 8 then d:setPixel(i-1,j-1,0,0,0,0) end
		end
		for j = h-6,h do
			if (h-j)+i < 8 then d:setPixel(i-1,j-1,0,0,0,0) end
		end
	end
	for i = w-6,w do
		for j = 1,6 do
			if (w-i)+j < 8 then d:setPixel(i-1,j-1,0,0,0,0) end
		end
		for j = h-6,h do
			if (h-j)+(w-i) < 8 then d:setPixel(i-1,j-1,0,0,0,0) end
		end
	end
	--top and bottom non-pyramidal
	d:setPixel(7-1,h-1,0,0,0,0)
	d:setPixel(8-1,h-1,0,0,0,0)
	d:setPixel((w-7)-1,h-1,0,0,0,0)
	d:setPixel((w-8)-1,h-1,0,0,0,0)
	d:setPixel(7-1,0,0,0,0,0)
	d:setPixel(8-1,0,0,0,0,0)
	d:setPixel((w-7)-1,0,0,0,0,0)
	d:setPixel((w-8)-1,0,0,0,0,0)
	--left/right non-pyramidal
	d:setPixel(0,7-1,0,0,0,0)
	d:setPixel(0,8-1,0,0,0,0)
	d:setPixel(0,h-7-1,0,0,0,0)
	d:setPixel(0,h-8-1,0,0,0,0)
	d:setPixel(w-1,7-1,0,0,0,0)
	d:setPixel(w-1,8-1,0,0,0,0)
	d:setPixel(w-1,h-7-1,0,0,0,0)
	d:setPixel(w-1,h-8-1,0,0,0,0)
end

function isNullCard(data)	--tests card image data to see if it's null card or not.
	local r,g,b,a = data:getPixel(12,80)
	if r==62 and g==41 and b==24 and a==255 then
		r,g,b,a = data:getPixel(91,40)
		if r==163 and g==77 and b==42 and a==255 then
			return true
		end
	end
	return false
end

function writeDeck(name,cards,mana)
	local n2 = string.lower(name)
	local n = ''
	for l in n2:gmatch('%l') do
		n = n .. l
	end
	--n is now the name of the deck in only lowercase letters
	
	local d = lfs.newFile('decks/' .. n .. 'deck.txt')
	d:open('w')
	
	d:write('return {\n')
	d:write('\tname = "' .. name .. '",\n')
	d:write('\tmana = {w = ' .. tostring(mana.w) .. ',g = ' .. tostring(mana.g) .. ',r = ' .. tostring(mana.r) .. ',b = ' .. tostring(mana.b) .. ',k = ' .. tostring(mana.k) .. '},\n')
	d:write('\tcards = {\n')
	for i,c in ipairs(cards) do
		if i~=#cards then
			d:write('\t\t{name = "' .. c.name .. '", quantity = ' .. c.quantity .. '},\n')
		else
			d:write('\t\t{name = "' .. c.name .. '", quantity = ' .. c.quantity .. '}\n')
		end
	end
	d:write('\t}\n')
	d:write('}')
	
	d:close()
end








