
TextButton = class('TextButton')

function TextButton:initialize(s,x,y,f,c)
	assert(type(s)=='string', "text button was initialized without a string")
	assert(type(x)=='number' and type(y)=='number', "text button was initialized without a numeric x and y coordinate")
	self.text = s
	self.x = x
	self.y = y
	self.color = c or {180,170,150,255}
	self.hoverColor = brightness(self.color,50)
	self.hover = false
	self.click = f or function() end
end

function TextButton:update()
	self:isOver(lm.getPosition())
end

function TextButton:draw()
	if self.hover then lg.setColor(self.hoverColor) else lg.setColor(self.color) end
	lg.print(self.text,self.x-magik32:getWidth(self.text)/2, self.y)
end

function TextButton:isOver(mx,my)
	if mx > self.x-magik32:getWidth(self.text)/2 and mx < self.x+magik32:getWidth(self.text)/2 and my > self.y and my < self.y+34 then
		self.hover = true
		return true
	else
		self.hover = false
		return false
	end
end

---------------------------------------------------------------------

TextField = class('TextField')

function TextField:initialize(x,y,w,h)
	self.rect = {x,y,w,h}
	self.text = ''
	self.focus = false
end

function TextField:update(dt)
	
end

function TextField:setFocus(x,y)
	local r = self.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		self.focus = true
	else
		self.focus = false
	end
end

function TextField:add(k,uc)
	if (uc >= 65 and uc <= 90) or (uc >= 97 and uc <= 122) then self.text = self.text .. string.char(uc) end
	if key==' ' or uc==32 then self.text = self.text .. ' ' end
	if (k=='backspace' or uc==8) then self.text = string.sub(self.text,1, self.text:len()-1) end
	if (k=='?' or uc==63) then self.text = self.text .. '?'
	elseif key==',' or uc==44 then self.text = self.text .. ','
	elseif key=='"' or uc==34 then self.text = self.text .. '"'
	elseif key=='!' or uc==33 then self.text = self.text .. '!'
	elseif key=='+' or uc==43 then self.text = self.text .. '+'
	elseif key=='(' or uc==40 then self.text = self.text .. '('
	elseif key==')' or uc==41 then self.text = self.text .. ')'
	elseif key=="'" or uc==39 then self.text = self.text .. "'"
	elseif key=='-' or uc==45 then self.text = self.text .. '-'
	end
end

function TextField:draw()
	lg.setColor(40,40,40)
	lg.rectangle('fill',self.rect[1]-3,self.rect[2]-2,self.rect[3]+6,self.rect[4]+4)
	lg.setColor(55,55,55)
	lg.rectangle('fill',self.rect[1],self.rect[2],self.rect[3],self.rect[4])
	lg.setFont(magik24)
	lg.setColor(beige)
	lg.print(self.text,self.rect[1]+15,self.rect[2]+3)
	if love.timer.getTime()%1 > .5 and self.focus then lg.line(self.rect[1]+15+magik24:getWidth(self.text)+3,self.rect[2]+6,self.rect[1]+15+magik24:getWidth(self.text)+3,self.rect[2]+self.rect[4]-6) end
end


--------------------------------------------------

CardBox = class('CardBox')

function CardBox:initialize(x,y,w,h)
	self.cards = {}
	self.rect = {x,y,w,h}
	self.scroller = {xx = x+w-9, yy = y+5 , miny = y+5, maxy = y+5+h-5, height = h-10 ,grabbed = false}
end

function CardBox:update(dt)
	
end

function CardBox:addCard(cardName)
	assert(type(cardName)=='string', "Please enter a string for the name of the card to be added.")
	local pic = getCardImage(cardName)
	local ed = states[#states]
	if pic then
		ed.cardPic = pic
		local found = false
		for i,c in ipairs(self.cards) do
			if c.name == cardName then found = c end
		end
		if found then
			found.quantity = found.quantity+1
		else
			table.insert(self.cards,{name = cardName, quantity = 1})
		end
	else
		ed.cardPic = img.nullcard
	end
end

function CardBox:draw()
	local r = self.rect
	lg.setColor(40,40,40)
	lg.rectangle('fill',r[1]-3,r[2]-2,r[3]+6,r[4]+4)
	lg.setColor(20,20,20)
	lg.rectangle('fill',r[1],r[2],r[3],r[4])
	lg.setFont(goudy14)
	--set scissor box for printing card name
	lg.setScissor(r[1],r[2],r[3],r[4])
	for i,c in ipairs(self.cards) do
		lg.setColor(25+(10*(i%2)),25+(10*(i%2)),25+(10*(i%2)))
		if i==selectedCard then lg.setColor(60,60,60) end
		lg.rectangle('fill',r[1],r[2]+14*(i-1)+6,r[3],12)
		lg.setColor(beige)
		lg.print(c.name,r[1]+2,r[2]+14*(i-1)+2)
		lg.print('x ' .. c.quantity,r[1]+350,r[2]+14*(i-1)+2)
	end
	--unset render bounds
	lg.setScissor()
end

function CardBox:isOver(x,y)	--returns card or nil
	local r = self.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		--clicked in list, but which card?
		for i,c in ipairs(self.cards) do
			if y >= r[2]+14*(i-1)+6 and y <= r[2]+14*(i-1)+6+12 then return i end
		end
	end
	return nil
end

-----------------------------------------------------------------

LoadDeckBox = class('LoadDeckBox')

function LoadDeckBox:initialize(x,y,w,h)
	self.rect = {x,y,w,h}
	self.decks = {}
	self.selectedDeck = nil
	
	for i,df in ipairs(lfs.enumerate('decks')) do
		table.insert(self.decks, lfs.load('decks/' .. df)())
	end
end

function LoadDeckBox:update(dt)

end

function LoadDeckBox:draw()
	local r = self.rect
	lg.setColor(40,40,40)
	lg.rectangle('fill',r[1]-3,r[2]-2,r[3]+6,r[4]+4)
	lg.setColor(20,20,20)
	lg.rectangle('fill',r[1],r[2],r[3],r[4])
	lg.setFont(goudy14)
	--set scissor box for printing deck name
	lg.setScissor(r[1],r[2],r[3],r[4])
	for i,d in ipairs(self.decks) do
		lg.setColor(25+(10*(i%2)),25+(10*(i%2)),25+(10*(i%2)))
		if i==self.selectedDeck then lg.setColor(60,60,60) self.flag = true end
		lg.rectangle('fill',r[1],r[2]+14*(i-1)+6,r[3],12)
		lg.setColor(beige)
		lg.print(d.name,r[1]+2,r[2]+14*(i-1)+2)
		lg.setColor(white)
		--use symbols
		if not d.mana.w then lg.draw(img.wSymbolbw,r[1]+r[3]-80,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) else lg.draw(img.wSymbol,r[1]+r[3]-80,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) end
		if not d.mana.g then lg.draw(img.gSymbolbw,r[1]+r[3]-65,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) else lg.draw(img.gSymbol,r[1]+r[3]-65,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) end
		if not d.mana.r then lg.draw(img.rSymbolbw,r[1]+r[3]-50,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) else lg.draw(img.rSymbol,r[1]+r[3]-50,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) end
		if not d.mana.b then lg.draw(img.bSymbolbw,r[1]+r[3]-35,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) else lg.draw(img.bSymbol,r[1]+r[3]-35,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) end
		if not d.mana.k then lg.draw(img.kSymbolbw,r[1]+r[3]-20,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) else lg.draw(img.kSymbol,r[1]+r[3]-20,r[2]+14*(i-1)+6+6,0,(1/28.6),(1/28.6),(286/2),(286/2)) end
		--use colored circles
		-- if d.mana.w then lg.setColor(255,255,255) lg.circle('fill', r[1]+r[3]-80,r[2]+14*(i-1)+6+6,5,16) else lg.setColor(30,30,30) lg.circle('fill',r[1]+r[3]-80,r[2]+14*(i-1)+6+6,5,16) end
		-- if d.mana.g then lg.setColor(0,255,0) lg.circle('fill', r[1]+r[3]-65,r[2]+14*(i-1)+6+6,5,16) else lg.setColor(0,40,0) lg.circle('fill',r[1]+r[3]-65,r[2]+14*(i-1)+6+6,5,16) end
		-- if d.mana.r then lg.setColor(255,0,0) lg.circle('fill', r[1]+r[3]-50,r[2]+14*(i-1)+6+6,5,16) else lg.setColor(40,0,0) lg.circle('fill',r[1]+r[3]-50,r[2]+14*(i-1)+6+6,5,16) end
		-- if d.mana.b then lg.setColor(0,0,255) lg.circle('fill', r[1]+r[3]-35,r[2]+14*(i-1)+6+6,5,16) else lg.setColor(0,0,40) lg.circle('fill',r[1]+r[3]-35,r[2]+14*(i-1)+6+6,5,16) end
		-- if d.mana.k then lg.setColor(120,120,120) lg.circle('fill', r[1]+r[3]-20,r[2]+14*(i-1)+6+6,5,16) else lg.setColor(20,20,20) lg.circle('fill',r[1]+r[3]-20,r[2]+14*(i-1)+6+6,5,16) end
	end
	--unset render bounds
	lg.setScissor()
end

function LoadDeckBox:isOver(x,y)
	local r = self.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		--clicked in list, but which deck?
		for i,d in ipairs(self.decks) do
			if y >= r[2]+14*(i-1)+6 and y <= r[2]+14*(i-1)+6+12 then return i end
		end
	end
	return nil
end

-------------------------------------------------------------------------------------------

IconButton = class('IconButton')

function IconButton:initialize(icon,x,y,f,w,h)
	self.icon = icon
	self.size = .25
	self.rect = {x,y,w or self.size*(114+30),h or self.size*(64+30)}
	self.click = f or function() end
	self.hover = false
	
end

function IconButton:update()
	self:isOver(lm.getPosition())
end

function IconButton:draw()
	local r = self.rect
	if self.hover then lg.setColor(white) else lg.setColor(255,255,255,80) end
	lg.draw(self.icon,r[1]+r[3]/2,r[2]+r[4]/2,0,self.size,self.size,self.icon:getWidth()/2,self.icon:getHeight()/2)
	local r = self.rect
	lg.rectangle('line',r[1],r[2],r[3],r[4])
end

function IconButton:isOver(mx,my)
	local r = self.rect
	if mx > r[1] and mx < r[1]+r[3] and my > r[2] and my < r[2]+r[4] then
		self.hover = true
		return true
	else
		self.hover = false
		return false
	end
end

-------------------------------------------------------------------------------------------

Slider = class('Slider')

function Slider:initialize(x,y,length,style)
	self.style = style or 'vertical'
	self.x = x or 20
	self.y = y or 20
	self.length = length or 100
	self.handle = 0
	self.hover = false
	self.held = false
end

function Slider:update(dt)
	self:isOver(mx,my)
	if self.held and lm.isDown('l') then
		if self.style =='vertical' then
			self.handle = clamp(0,my-self.y,self.length)
		else
			self.handle = clamp(0,mx-self.x,self.length)
		end
	end
end

function Slider:draw()
	lg.setColor(255,255,255,120)
	if self.style=='vertical' then
		lg.setLineWidth(2)
		lg.line(self.x,self.y,self.x,self.y+self.length)
		lg.setLineWidth(1)
		if self.hover or self.held then
			lg.rectangle('fill',self.x-3,self.y+self.handle-5,6,10)
		else
			lg.rectangle('line',self.x-3,self.y+self.handle-5,6,10)
		end
	else
		lg.setLineWidth(2)
		lg.line(self.x,self.y,self.x+self.length,self.y)
		lg.setLineWidth(1)
		if self.hover or self.held then
			lg.rectangle('fill',self.x-5+self.handle,self.y-3,10,6)
		else
			lg.rectangle('line',self.x-5+self.handle,self.y-3,10,6)
		end
	end
end

function Slider:isOver(xx,yy)
	if self.style == 'vertical' then
		if xx >= self.x-3 and xx <= self.x+3 and yy >= self.y+self.handle-5 and yy <= self.y+self.handle+5 then
			self.hover = true
			return true
		else
			self.hover = false
			return false
		end
	else
		if xx >= self.x+self.handle-5 and xx <= self.x+self.handle+5 and yy >= self.y-3 and yy <= self.y+3 then
			self.hover = true
			return true
		else
			self.hover = false
			return false
		end
	end
end












