--math/physics/etc functions written by me

math.randomseed(os.time())
math.random()
math.random()
math.random()

function coll(box1, box2)	--box1 and box2 must be tables with values { x1, y1, width, height }
	ax1, ay1, aw, ah, bx1, by1, bw, bh = box1[1], box1[2], box1[3], box1[4], box2[1], box2[2], box2[3], box2[4]
	local ax2,ay2,bx2,by2 = ax1 + aw, ay1 + ah, bx1 + bw, by1 + bh
	return ax1 < bx2 and ax2 > bx1 and ay1 < by2 and ay2 > by1
end

function collSets(set1, set2)	--see if any rectangle in one set collides with any rectangle in another
	for i = 1, #set1 do
		for j = 1, #set2 do
			if coll(set1[i], set2[j]) then return true end
		end
	end
	return false
end
	
function round(x)
	if x == 0 then return 0 end
	if x > 0 then
		if x%1 < 0.5 then return (x-x%1)
		else return (x+ 1-x%1) end
	else
		if x%1 < 0.5 then return math.floor(x)
		else return math.ceil(x) end
	end
end

function ordersOfMag(x)
	if x == 0 then return x end
	if x < 0 then x = x*-1 end
	for i = 1, 1000 do
		if x < 10^i then return i end
	end
	return 1001
end

function shuffle(t) 	--returns shuffled copy of table t
	local rt = {}
	local k = 0
	while #t > 0 do
		k = math.random(1,#t)
		table.insert(rt,t[k])
		table.remove(t,k)
	end
	return rt
end

function loadResources(a, b)

	local base_folder, resource_table

	if type(a) == 'string' then
		base_folder = a
	elseif type(b) == 'string' then
		base_folder = b
	else
		base_folder = ''
	end

	if type(a) == 'table' then
		resource_table = a
	elseif type(b) == 'table' then
		resource_table = b
	else
		resource_table = _G
	end

	local function load_directory(folder, place)

		for _, item in pairs(love.filesystem.enumerate(folder)) do

			local path = folder..'/'..item

			if love.filesystem.isFile(path) then
				local name, ext = item:match('(.+)%.(.+)$')

				if ext == 'png' or ext == 'bmp' or ext == 'jpg' or ext == 'jpeg' or ext == 'gif' or ext == 'tga' then
					place[name] = love.graphics.newImage(path)
					place[name]:setFilter('linear' , 'linear')

				elseif ext == 'ogg' or ext == 'mp3' or ext == 'wav' or ext == 'xm' or ext == 'it' or ext == 's3m' then
					place[name] = love.audio.newSource(path)
				end

			else
				place[item] = {}
				load_directory(path, place[item])
			end
		end
	end

	load_directory(base_folder, resource_table)

end

--circle: x = r*math.cos(rads)		y = r*math.sin(rads)

function inCircle(x, y, cx, cy, cr)
    local dx = cx - x
    local dy = cy - y
    return dx^2 + dy^2 < cr^2
end

function clamp(low, val, high)
	return math.max(low, math.min(val,high))
end

function brightness(ct,val)
	assert(ct,'brightness function requires color table to be formatted {rval,gval,bval,aval}')
	local c = {}
	for i,v in ipairs(ct) do
		c[i] = v
	end
	for i=1,#c do
		c[i]=clamp(0,c[i]+val,255)
	end
	return c
end

function imageFromUrl(url)
   return love.graphics.newImage(
          love.image.newImageData(
          love.filesystem.newFileData(
          http.request(url), '', 'file')))
end


