--The functions are the first half of implementing states in your game.
------------------------------------------------------State functions

function pushState(state)
	table.insert(states,state)
	state:setCallbacks()
end

function switchState(state)	--switches with the state on top of the stack
	popState()
	pushState(state)
end

function popState()
	states[#states]:destroy()
	table.remove(states,#states)
	if next(states) then
		states[#states]:setCallbacks()
	end
end

function updateStates(dt)
	for i,state in ipairs(states) do
		if i==#states then
			state:update(dt)
		else
			if state.alwaysUpdate then state:update(dt) end
		end
	end
end

function drawStates()
	for i,state in ipairs(states) do
		if i==#states then
			state:draw()
		else
			if state.alwaysDraw then state:draw() end
		end
	end
end

-------------------------------------------------------State Classes
--[[
There are two ways to implement the actual classes:
1.) You can have a superclass that has the needed functions and members defined and then make subclasses that inherit/override those members and functions
2.) You can make StateClasses and just make sure they always have the appropriate members/methods.

Regardless, if you create an instance of a class, and intend to use it with the state functions and global states table, make sure it has members:
alwaysUpdate = true/false
alwaysDraw = true/false

and make sure it has methods:

update
draw
setCallbacks
destroy

I prefer the second method, so my state classes might look like:
]]

TitleState = class('TitleState')

function TitleState:initialize()
	self.alwaysUpdate = true
	self.alwaysDraw = true
	
	--ring determines ring parameters
	self.ring = { speed = (1/(2000*math.pi)), radius = 250, symbolSize = .25, opacity = 255, x = sw/2, y = sh/2, r = (4/5)*2*math.pi}
	lg.setFont(magik32)
	
	--make, organize buttons
	
	self.buttons = {
	newB = TextButton('New Deck',sw/2,sh/2-75,function() editor = EditState() switchState(editor) end),
	editB = TextButton('Open Deck',sw/2,sh/2-35,function() loadState = LoadState() pushState(loadState) end),
	quitB = TextButton('Quit', sw/2,sh/2+5, function() love.event.quit() end)
	}
end

function TitleState:update(dt)
	--update buttons
	for i,b in pairs(self.buttons) do
		b:update()
	end
	
	--spin ring
	self.ring.r = self.ring.r+self.ring.speed
	
end

function TitleState:draw()
	--draw buttons
	lg.setFont(magik32)
	for i,b in pairs(self.buttons) do
		b:draw()
	end
	
	--draw ring
	lg.setColor(20,20,20)
	lg.circle('line',sw/2,sh/2,200,64)
	-- lg.circle('line',sw/2,sh/2,295,14)
	lg.setColor(255,255,255)
	lg.draw(img.wSymbol, (self.ring.x+self.ring.radius*math.cos(self.ring.r)), (self.ring.y+self.ring.radius*math.sin(self.ring.r)), self.ring.r, self.ring.symbolSize, self.ring.symbolSize, 286/2,286/2)
	lg.draw(img.gSymbol, (self.ring.x+self.ring.radius*math.cos(self.ring.r+(1/5)*2*math.pi)), (self.ring.y+self.ring.radius*math.sin(self.ring.r+(1/5)*2*math.pi)), (self.ring.r+(9/10)*math.pi), self.ring.symbolSize, self.ring.symbolSize, 286/2,286/2)
	lg.draw(img.rSymbol, (self.ring.x+self.ring.radius*math.cos(self.ring.r+(2/5)*2*math.pi)), (self.ring.y+self.ring.radius*math.sin(self.ring.r+(2/5)*2*math.pi)), (self.ring.r+(7/10)*math.pi), self.ring.symbolSize, self.ring.symbolSize, 286/2,286/2)
	lg.draw(img.bSymbol, (self.ring.x+self.ring.radius*math.cos(self.ring.r+(3/5)*2*math.pi)), (self.ring.y+self.ring.radius*math.sin(self.ring.r+(3/5)*2*math.pi)), (self.ring.r+(-3/9)*math.pi), self.ring.symbolSize, self.ring.symbolSize, 286/2,286/2)
	lg.draw(img.kSymbol, (self.ring.x+self.ring.radius*math.cos(self.ring.r+(4/5)*2*math.pi)), (self.ring.y+self.ring.radius*math.sin(self.ring.r+(4/5)*2*math.pi)), (self.ring.r+(1/10)*math.pi), self.ring.symbolSize, self.ring.symbolSize, 286/2,286/2)
	
	--draw Title
	lg.print('Magic:',sw/2-magik32:getWidth('Magic:')/2,sh/2-18-130)
	lg.print('The Deck Editor', sw/2-magik32:getWidth('The Deck Editor')/2, sh/2+18-130)
end

function TitleState:destroy()
	
end

function TitleState:setCallbacks()
	function love.keypressed(k,uc)
		if k == 'esc' then
			
		end
	end
	
	function love.mousepressed(x,y,bttn)
		for i,b in pairs(self.buttons) do
			if b:isOver(x,y) and bttn=='l' then
				b:click()
			end
		end
	end
end

----------------------------------------------------------------------------------------------------

EditState = class('EditState')

function EditState:initialize(loadedDeck)
	self.alwaysUpdate = false
	self.alwaysDraw = true
	
	self.cardPic = img.nullcardgray
	self.cardCount = 0
	self.saving = false
	--buttons
	backB = TextButton('Back', sw-130,sh-20-32,function() title = TitleState() switchState(title) end)
	playB = TextButton('Play', sw-50,sh-20-32,function() deck = {name = deckNameField.text, cards = cardBox.cards, mana = mana} playS = PlayState() states = {} pushState(playS) end)
	saveB = TextButton('Save', sw-130,20, function() states[#states].saving = true end)
	loadB = TextButton('Load', sw-50,20,function() loadState = LoadState() pushState(loadState) end)
	okayB = TextButton('Okay', sw/2+100-magik24:getWidth('Okay')/2,sh/2+55,function() local ed = states[#states] writeDeck(deckNameField.text, cardBox.cards, mana) ed.saving = false end)
	cancelB = TextButton('Cancel', sw/2-100+magik24:getWidth('Cancel')/2,sh/2+55,function() states[#states].saving = false end)
	checkB = TextButton('Check', sw-215,458,function() local pic = getCardImage(cardAddField.text) if pic then states[#states].cardPic = pic end end)
	addB = TextButton('Add', sw-90,458,function() cardBox:addCard(cardAddField.text) end)
	deckNameField = TextField(magik32:getWidth('Deck Name:')+30,20,435,32)
	mana = {w = false, g = false, r = false, b = false, k = false}
	cardAddField = TextField(135,455,345,32)
	cardAddField.focus = true
	cardBox = CardBox(20,110,460,330)
	selectedCard = nil
end

function EditState:update(dt)
	self.cardCount = 0
	backB:update()
	saveB:update()
	loadB:update()
	addB:update()
	checkB:update()
	playB:update()
	okayB:update()
	cancelB:update()
	for i,c in ipairs(cardBox.cards) do
		self.cardCount = self.cardCount + c.quantity
	end
end

function EditState:draw()
	--draw cardBox
	cardBox:draw()
	
	--draw card picture slot/nullcardgray
	lg.setColor(255,255,255)
	lg.draw(self.cardPic,20+460+45,120)
	
	--draw buttons, textfields..
	lg.setFont(magik32)
	backB:draw()
	playB:draw()
	saveB:draw()
	loadB:draw()
	checkB:draw()
	addB:draw()
	deckNameField:draw()
	cardAddField:draw()
	
	--draw static text, labels, etc.
	lg.setColor(beige)
	lg.print('Deck Name:',20,20)
	lg.print('Card List:',20,75)
	lg.setFont(magik24)
	lg.print('Add Card:',20,458)
	lg.print(self.cardCount, 370,80)
	-- lg.print(tostring(selectedCard),5,580)
	
	if self.saving then
		lg.setColor(0,0,0,220)
		lg.rectangle('fill',0,0,sw,sh)
		lg.setColor(20,20,20)
		lg.rectangle('fill',sw/2-203,sh/2-102,406,204)
		lg.setColor(0,0,0)
		lg.rectangle('fill',sw/2-200,sh/2-100,400,200)
		lg.setFont(magik24)
		okayB:draw()
		cancelB:draw()
		lg.setColor(beige)
		lg.print('Which mana does this deck use?',sw/2-magik24:getWidth('Which mana does this deck use?')/2,sh/2-85)
		if mana.w then lg.setColor(white) lg.draw(img.wSymbol,sw/2-140,sh/2-30,0,.15,.15,286/2) else lg.setColor(255,255,255,75) lg.draw(img.wSymbolbw,sw/2-140,sh/2-30,0,.15,.15,286/2) end
		if mana.g then lg.setColor(white) lg.draw(img.gSymbol,sw/2-70,sh/2-30,0,.15,.15,286/2) else lg.setColor(255,255,255,75) lg.draw(img.gSymbolbw,sw/2-70,sh/2-30,0,.15,.15,286/2) end
		if mana.r then lg.setColor(white) lg.draw(img.rSymbol,sw/2,sh/2-30,0,.15,.15,286/2) else lg.setColor(255,255,255,75) lg.draw(img.rSymbolbw,sw/2,sh/2-30,0,.15,.15,286/2) end
		if mana.b then lg.setColor(white) lg.draw(img.bSymbol,sw/2+70,sh/2-30,0,.15,.15,286/2) else lg.setColor(255,255,255,75) lg.draw(img.bSymbolbw,sw/2+70,sh/2-30,0,.15,.15,286/2) end
		if mana.k then lg.setColor(white) lg.draw(img.kSymbol,sw/2+140,sh/2-30,0,.15,.15,286/2) else lg.setColor(255,255,255,75) lg.draw(img.kSymbolbw,sw/2+140,sh/2-30,0,.15,.15,286/2) end
	end
end

function EditState:destroy()
	
end

function EditState:setCallbacks()
	function love.mousepressed(x,y,b)
		if states[#states].saving then
			local z = .15*(286/2)
			if cancelB:isOver(x,y) and b=='l' then cancelB:click() end
			if okayB:isOver(x,y) and b=='l' then okayB:click() end
			if x > sw/2-140-z and x < sw/2-140+z and y < sh/2-30+z+z and y > sh/2-30 then mana.w = not mana.w end
			if x > sw/2-70-z and x < sw/2-70+z and y < sh/2-30+z+z and y > sh/2-30 then mana.g = not mana.g end
			if x > sw/2-z and x < sw/2+z and y < sh/2-30+z+z and y > sh/2-30 then mana.r = not mana.r end
			if x > sw/2+70-z and x < sw/2+70+z and y < sh/2-30+z+z and y > sh/2-30 then mana.b = not mana.b end
			if x > sw/2+140-z and x < sw/2+140+z and y < sh/2-30+z+z and y > sh/2-30 then mana.k = not mana.k end
		else
			if backB:isOver(x,y) and b=='l' then backB:click() end
			if playB:isOver(x,y) and b=='l' then playB:click() end
			if saveB:isOver(x,y) and b=='l' then saveB:click() end
			if loadB:isOver(x,y) and b=='l' then loadB:click() end
			if checkB:isOver(x,y) and b=='l' then checkB:click() end
			if addB:isOver(x,y) and b=='l' then addB:click() end
			deckNameField:setFocus(x,y)
			cardAddField:setFocus(x,y)
			local cIndex = cardBox:isOver(x,y)
			selectedCard = cIndex
			if cIndex then states[#states].cardPic = getCardImage(cardBox.cards[cIndex].name) end
		end
	end
	
	function love.keypressed(k,uc)
		if deckNameField.focus and deckNameField.text:len() < 63 then deckNameField:add(k,uc) end
		if cardAddField.focus and (k=='enter' or uc == 13) then cardBox:addCard(cardAddField.text) end
		if cardAddField.focus and cardAddField.text:len() < 127 then cardAddField:add(k,uc) end
		if selectedCard~=nil and (k=='backspace' or k=='delete') then
			local c = cardBox.cards[selectedCard]
			if c.quantity~=nil and c.quantity > 1 then
				c.quantity = c.quantity -1
			else
				table.remove(cardBox.cards,selectedCard)
			end
		end
	end
end

function EditState:loadDeck()
	cardBox.cards = deck.cards
	deckNameField.text = deck.name
	mana = deck.mana
	-- self.cardPic = getCardImage(deck.cards[1].name) or img.nullcardgray
end

-------------------------------------------------------------------------

MenuState = class('MenuState')

function MenuState:initialize()
	self.alwaysUpdate = true
	self.alwaysDraw = true
	
	self.buttons = {
		keepPlaying = TextButton('Keep Playing', sw/2, sh/2-(24*3)/2-56, function() popState() end),
		openDeck = TextButton('Open Deck',sw/2,sh/2-(24*3)/2-10,function() loadS = LoadState() switchState(loadS) end),
		newDeck = TextButton('New Deck',sw/2,sh/2, function() deck = nil editor = EditState() states = {} pushState(editor) end),
		quit = TextButton('Quit',sw/2,sh/2+(24*3)/2+10, function() title = TitleState() states = {} pushState(title) end)
	}
end

function MenuState:update()
	for i,b in pairs(self.buttons) do
		b:update()
	end
end

function MenuState:draw()
lg.setColor(0,0,0,220)
	lg.rectangle('fill',0,0,sw,sh)
	lg.setColor(20,20,20)
	lg.rectangle('fill',sw/2-203,sh/2-112,406,224)
	lg.setColor(0,0,0)
	lg.rectangle('fill',sw/2-200,sh/2-110,400,220)
	
	lg.setFont(magik32)
	lg.setColor(beige)
	for i,b in pairs(self.buttons) do
		b:draw()
	end
end

function MenuState:destroy()

end

function MenuState:setCallbacks()
	function love.mousepressed(x,y,z)
		for i,b in pairs(self.buttons) do
			if b:isOver(x,y) and z=='l' then b:click() end
		end
	end
end

----------------------------------------------------------------------------------------------------

LoadState = class('LoadState')

function LoadState:initialize()
	self.alwaysUpdate = true
	self.alwaysDraw = true
	
	self.loadB = TextButton('Edit',sw/2+10,sh/2+55,function()
		if loadDeckBox.selectedDeck then
			deck = loadDeckBox.decks[loadDeckBox.selectedDeck]
			if instanceOf(EditState,states[#states-1]) then
				states[#states-1].loadDeck()
				popState()
			else
				editState = EditState()
				editState:loadDeck()
				states = {}
				pushState(editState)
			end
		end
	end)
	self.cancelB = TextButton('Cancel',sw/2-100,sh/2+55,function() popState() end)
	self.playB = TextButton('Test',sw/2+115,sh/2+55,function() if loadDeckBox.selectedDeck then deck = loadDeckBox.decks[loadDeckBox.selectedDeck] playS = PlayState() states = {} pushState(playS) end end)
	loadDeckBox = LoadDeckBox(sw/2-185,sh/2-90,185*2,130,{})
end

function LoadState:update(dt)
	self.cancelB:update()
	self.loadB:update()
	self.playB:update()
	loadDeckBox:update(dt)
end

function LoadState:draw()
	lg.setColor(0,0,0,220)
	lg.rectangle('fill',0,0,sw,sh)
	lg.setColor(20,20,20)
	lg.rectangle('fill',sw/2-203,sh/2-102,406,204)
	lg.setColor(0,0,0)
	lg.rectangle('fill',sw/2-200,sh/2-100,400,200)
	lg.setColor(beige)
	lg.setFont(magik24)
	self.loadB:draw()
	self.cancelB:draw()
	self.playB:draw()
	loadDeckBox:draw()
	-- lg.setColor(white)
	-- lg.print(type(loadDeckBox.decks[loadDeckBox.selectedDeck]),20,40)
end

function LoadState:destroy()
	
end

function LoadState:setCallbacks()
	function love.mousepressed(x,y,b)
		if self.loadB:isOver(x,y) and b=='l' then self.loadB:click() end
		if self.playB:isOver(x,y) and b=='l' then self.playB:click() end
		if self.cancelB:isOver(x,y) and b=='l' then self.cancelB:click() end
		loadDeckBox.selectedDeck = loadDeckBox:isOver(x,y)
	end
	function love.keypressed(k,uc)
		if (k=='delete' or k=='backspace') and loadDeckBox.selectedDeck~=nil then
			local n2 = string.lower(loadDeckBox.decks[loadDeckBox.selectedDeck].name)
			local n = ''
			for l in n2:gmatch('%l') do
				n = n .. l
			end
			lfs.remove('decks/' .. n .. 'deck.txt')
			table.remove(loadDeckBox.decks,loadDeckBox.selectedDeck)
		end
	end
end

--------------------------------------------------------------------------------------

PlayState = class('PlayState')

function PlayState:initialize()
	self.alwaysUpdate = true
	self.alwaysDraw = true
	
	self.cardSize = math.min(sw/(223*8), sh/(310*4))
	self.life = 20
	self.manaCount = {0,0,0,0,0}
	self.searching = nil
	self.searchCard = nil
	self.libSlider = Slider(sw*.25,sh*.15,sw*.5,'horizontal')
	self.reveal = 0
	self.libSlider.handle = sw*.5
	self.graveSlider = Slider(sw*.25,sh*.15,sw*.5,'horizontal')
	self.graveSlider.handle = sw*.5
	--set field areas
	self.library = {rect = {(15/800)*sw,0.4*sh,self.cardSize*223,self.cardSize*310}, cards = {}}
	self.graveyard = {rect = {sw-sw*(15/800)-self.cardSize*223, .4*sh, self.cardSize*223, self.cardSize*310}, cards = {}}
	self.hand = {rect = {0,.4*sh+self.cardSize*310+5+72,sw,sh-(.4*sh+self.cardSize*310+5+80)}, cards = {}}
	self.field = {rect = {(15/800)*sw*2+self.cardSize*223,sh*.05,sw-2*((15/800)*sw*2+self.cardSize*223), sh-self.hand.rect[4]-sh*.10}, cards = {}}
	
	self.buttons = {--			(icon				x												y									function																width )
		shuffleL = IconButton(img.shuffleIcon,		(15/800)*sw+1									,0.4*sh+310*self.cardSize+15, 		function() self.library.cards = shuffle(self.library.cards) end			,self.cardSize*223/2-8),
		searchL = IconButton(img.eyeIcon,			(15/800)*sw+1+self.cardSize*223/2+6				,0.4*sh+310*self.cardSize+15,		function() self.searching = self.library.cards end						,self.cardSize*223/2-8),
		shuffleG = IconButton(img.shuffleIcon,		sw-sw*(15/800)-self.cardSize*223+1				,0.4*sh+310*self.cardSize+15, 		function() self.graveyard.cards = shuffle(self.graveyard.cards) end		,self.cardSize*223/2-8),
		searchG = IconButton(img.eyeIcon,			sw-sw*(15/800)-self.cardSize*223/2+6			,0.4*sh+310*self.cardSize+15,		function() self.searching = self.graveyard.cards end					,self.cardSize*223/2-8),
		menu = IconButton(img.menuIcon,sw*.02,sh*.02,function() menuS = MenuState() pushState(menuS) end)
	}
	self.sButtons = {
		revOne = IconButton(img.revealIcon			,sw/2-50										,sh*.8								,function() self.reveal = clamp(0,self.reveal + 1,#self.searching) self.libSlider.handle = ((#self.searching-self.reveal+1)/#self.searching)*self.libSlider.length end,100),
		revAll = IconButton(img.revAllIcon			,sw/2-50-150									,sh*.8								,function() self.reveal = #self.library.cards end,100),
		back = IconButton(img.backIcon				,sw/2-50+150									,sh*.8								,function() self.searching = nil self.searchCard = nil self.reveal = 0 end, 100)		
	}
	self.scButtons = {
		hand = IconButton(img.handIcon				,sw/2-50-75										,sh*.88								,function() if self.searchCard then self:cardFromTo(self.searchCard,self.searching,self.hand.cards) self.reveal = self.reveal - 1 self.searchCard = nil end end,100),
		grave = IconButton(img.graveIconSmall		,sw/2-50+75										,sh*.88								,function() if self.searchCard then self:cardFromTo(self.searchCard,self.searching,self.graveyard.cards) self.reveal = self.reveal - 1 self.searchCard = nil end end, 100)
	}
	self.gButtons = {
		back = IconButton(img.back2Icon				,sw/2-50										,sh*.8								,function() self.searching = nil self.searchCard = nil end,100)
	}
	self.gcButtons = {
		under = IconButton(img.underIcon			,sw/2-50-150										,sh*.88								,function() if self.searchCard then table.insert(self.library.cards,self.searching[self.searchCard],1) table.remove(self.searching,self.searchCard) self.searchCard = nil end end, 100),
		hand  = IconButton(img.handIcon				,sw/2-50											,sh*.88								,function() if self.searchCard then self:cardFromTo(self.searchCard,self.searching,self.hand.cards) self.searchCard = nil end end, 100),
		over  = IconButton(img.overIcon				,sw/2-50+150										,sh*.88								,function() if self.searchCard then self:cardFromTo(self.searchCard,self.searching,self.library.cards) self.searchCard = nil end end, 100)
	}
	
	--put deck cards in library
	for i,c in ipairs(deck.cards) do
		for j = 1,c.quantity do
			table.insert(self.library.cards, Card(c.name))
		end
	end
	self.DECKSIZE = #self.library.cards
	--shuffle library
	self.buttons.shuffleL.click()
end

function PlayState:update(dt)
	for i,b in pairs(self.buttons) do
		b:update()
	end
	
	if self.searching then
		if self.searching == self.library.card then
			self.libSlider:update(dt)
			for i,b in pairs(self.sButtons) do
				b:update()
			end
			if self.searchCard then
				for i,b in pairs(self.scButtons) do
					b:update()
				end
			end
		else	--is graveyard
			self.graveSlider:update(dt)
			for i,b in pairs(self.gButtons) do
				b:update()
			end
			if self.searchCard then
				for i,b in pairs(self.gcButtons) do
					b:update()
				end
			end
		end
	end
	
	local g = self.grabbed
	if g then --drag grabbed card
		if g[2]==self.field.cards then
			g[1].x = g.oldx + (mx-g.fx)
			g[1].y = g.oldy + (my-g.fy)
		else
			g[1].x = mx
			g[1].y = my
		end
	end
end

function PlayState:draw()
	
	--draw ring
	
	
	--draw buttons
	for i,b in pairs(self.buttons) do
		b:draw()
	end

	--draw library
	local r = self.library.rect
	lg.setColor(20,20,20)
	lg.rectangle('line',r[1],r[2],r[3],r[4])
	lg.setColor(40,40,40)
	lg.draw(img.libIcon,r[1]+r[3]/2,r[2]+r[4]/2,0,1,1,img.libIcon:getWidth()/2,img.libIcon:getHeight()/2)
	if #self.library.cards~=0 then
		lg.setColor(white)
		lg.draw(img.nullcard,(15/800)*sw,sh*.4,0,self.cardSize,self.cardSize)
	end
	lg.setColor(255,255,255,100)
	lg.setFont(magik24)
	lg.print('(' .. tostring(#self.library.cards) .. '/' .. tostring(self.DECKSIZE) .. ')',r[1]+10,r[2]-28)
	
	--draw hand
	local r = self.hand.rect
	lg.setColor(20,20,20)
	lg.rectangle('line',r[1],r[2],r[3],r[4])
	lg.setColor(white)
	for i,c in ipairs(self.hand.cards) do
		if not self.grabbed then
			c:draw()
		else
			if i~=self.grabbed.ci or self.grabbed[2]~=self.hand.cards then
				c:draw()
			end
		end
	end
	
	--draw field
	local r = self.field.rect
	lg.setColor(20,20,20)
	lg.rectangle('line',r[1],r[2],r[3],r[4])
	lg.setColor(white)
	for i,c in ipairs(self.field.cards) do
		if not self.grabbed then
			c:draw()
		else
			if i~=self.grabbed.ci or self.grabbed[2]~=self.field.cards then
				c:draw()
			end
		end
	end
	
	--draw graveyard
	local r = self.graveyard.rect
	lg.setColor(20,20,20)
	lg.rectangle('line',r[1],r[2],r[3],r[4])
		--draw top and second top card
	if #self.graveyard.cards > 1 then
		lg.setColor(120,120,120)
		local c1 = self.graveyard.cards[#self.graveyard.cards-1]
		local c2 = self.graveyard.cards[#self.graveyard.cards]
		c1:draw()
		c2:draw()
	elseif #self.graveyard.cards == 1 then
		lg.setColor(40,40,40)
		lg.draw(img.graveIcon,r[1]+r[3]/2,r[2]+r[4]/2,0,1,1,img.graveIcon:getWidth()/2,img.graveIcon:getHeight()/2)
		lg.setColor(120,120,120)
		local c2 = self.graveyard.cards[#self.graveyard.cards]
		c2:draw()
	else
		lg.setColor(40,40,40)
		lg.draw(img.graveIcon,r[1]+r[3]/2,r[2]+r[4]/2,0,1,1,img.graveIcon:getWidth()/2,img.graveIcon:getHeight()/2)
	end
	
	--draw life
	lg.setColor(clamp(0,255-(self.life*(255/20)),255),clamp(0,self.life*(200/20),200),clamp(0,self.life*(50/3),50))
	lg.circle('line',(15/800)*sw+self.cardSize*223/2,sw*.18,sw*.05,32)
	lg.setFont(magik72)
	lg.print(tostring(self.life),(15/800)*sw+self.cardSize*223/2-magik72:getWidth(tostring(self.life))/2,sw*.18-90/2)
	
	--draw mana
	
	--draw grabbed card
	if self.grabbed then
		lg.setColor(white)
		self.grabbed[2][self.grabbed.ci]:draw()
	end
	
	--if searching, show search mini-state
	if self.searching then
		lg.setColor(0,0,0,200)
		lg.rectangle('fill',0,0,sw,sh)
		
		if self.searching == self.library.cards then
			self.libSlider:draw()
			lg.setColor(white)
			for i,b in pairs(self.sButtons) do
				b:draw()
			end
			if self.searchCard then
				for i,b in pairs(self.scButtons) do
					b:draw()
				end
			end
			lg.setColor(white)
			for i,c in ipairs(self.searching) do
				if i == self.searchCard then
					lg.setColor(white)
					lg.rectangle('line',sw/2+(i)*(223+25)-(self.libSlider.handle/self.libSlider.length)*(223+25)*#self.searching-.8*223/2-3,sh/2-.8*310/2-3,.8*223+6,.8*310+6)
				end
				if i > #self.searching-self.reveal then
					lg.draw(c.image,sw/2+(i)*(223+25)-(self.libSlider.handle/self.libSlider.length)*(223+25)*#self.searching,sh/2,0,.8,.8,223/2,310/2)
				else
					lg.draw(img.nullcard,sw/2+(i)*(223+25)-(self.libSlider.handle/self.libSlider.length)*(223+25)*#self.searching,sh/2,0,.8,.8,223/2,310/2)
				end
			end
		else	--graveyard
			lg.setColor(white)
			self.graveSlider:draw()
			for i,b in pairs(self.gButtons) do
				b:draw()
			end
			if self.searchCard then
				for i,b in pairs(self.gcButtons) do
					b:draw()
				end
			end
			lg.setColor(white)
			for i,c in ipairs(self.searching) do
				if i == self.searchCard then
					lg.setColor(white)
					lg.rectangle('line',sw/2+(i)*(223+25)-(self.graveSlider.handle/self.graveSlider.length)*(223+25)*#self.searching-.8*223/2-3,sh/2-.8*310/2-3,.8*223+6,.8*310+6)
				end
				lg.draw(c.image,sw/2+(i)*(223+25)-(self.graveSlider.handle/self.graveSlider.length)*(223+25)*#self.searching,sh/2,0,.8,.8,223/2,310/2)
			end
		end
	end
	
	--if zoomed, draw zoomed card
	if self.zoom then
		lg.setColor(0,0,0,200)
		lg.rectangle('fill',0,0,sw,sh)
		lg.setColor(white)
		lg.draw(self.zoom,sw/2,sh/2,0,4*self.cardSize,4*self.cardSize,223/2,310/2)
	end
end

function PlayState:destroy()
	
end

function PlayState:getArea(x,y)	--given mx,my, determine area - self.field, self.hand .... , or nil

	local r = self.library.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		return self.library
	end
	
	local r = self.field.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		return self.field
	end
	
	local r = self.hand.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		return self.hand
	end
	
	local r = self.graveyard.rect
	if x > r[1] and x < r[1]+r[3] and y > r[2] and y < r[2]+r[4] then
		return self.graveyard
	end
	
	return nil
end

function PlayState:adjustHand()
	local startX = sw/2-(#self.hand.cards-1)*40-80
	for i,c in ipairs(self.hand.cards) do
		c.x = startX+80*i
		c.y = sh*.85
	end
end

function PlayState:cardFromTo(cI,fromT,toT)
	assert(type(fromT)=='table', "tried to take card from non-table")
	assert(type(cI)=='number', "card index should be a number")
	assert(type(toT)=='table', "tried to put card in a non-table")
	table.insert(toT,fromT[cI])
	table.remove(fromT,cI)
	if toT==self.hand.cards or fromT==self.hand.cards then self:adjustHand() end
	if toT == self.graveyard.cards then
		local r = self.graveyard.rect
		toT[#toT].x = r[1]+r[3]/2
		toT[#toT].y = r[2]+r[4]/2
	end
	if fromT == self.field.cards then
		toT[#toT].size = self.cardSize
	end
	if toT == self.field.cards then
		toT[#toT].size = 0.8*self.cardSize
	end
end

function PlayState:setCallbacks()
	function love.mousepressed(xx,yy,z)
		
		if self.searching then
			if self.searching == self.library.cards then
				if self.libSlider:isOver(xx,yy) and z=='l' then self.libSlider.held = true end
				
				for i,b in pairs(self.scButtons) do
					if b:isOver(xx,yy) and z=='l' then b:click() end
				end
				for i = 1,#self.searching do
					local cx = sw/2+(i)*(223+25)-(self.libSlider.handle/self.libSlider.length)*(223+25)*#self.searching
					if xx > cx-.8*223/2 and xx < cx+.8*223/2 and yy > sh/2-.8*310/2 and yy < sh/2+.8*310/2 then
						self.searchCard = i
					end
				end
				for i,b in pairs(self.sButtons) do
					if b:isOver(xx,yy) and z=='l' then b:click() end
				end
			else	--graveyard
				if self.graveSlider:isOver(xx,yy) and z=='l' then self.graveSlider.held = true end
				
				for i,b in pairs(self.gcButtons) do
					if b:isOver(xx,yy) and z=='l' then b:click() end
				end
				for i = 1,#self.searching do
					local cx = sw/2+(i)*(223+25)-(self.graveSlider.handle/self.graveSlider.length)*(223+25)*#self.searching
					if xx > cx-.8*223/2 and xx < cx+.8*223/2 and yy > sh/2-.8*310/2 and yy < sh/2+.8*310/2 then
						self.searchCard = i
					end
				end
				for i,b in pairs(self.gButtons) do
					if b:isOver(xx,yy) and z=='l' then b:click() end
				end
			end
		else
			
			for i,b in pairs(self.buttons) do
				if b:isOver(xx,yy) and z=='l' then b:click() end
			end
			
			--right click on a field or hand card to zoom on a card, if not already zoomed, left-click to grab
			if not self.zoom then
				for i = 1,#self.hand.cards do
					local c = self.hand.cards[#self.hand.cards-i+1]
					if c:isOver(xx,yy) and z=='r' then self.zoom = c.image break end
					if c:isOver(xx,yy) and z=='l' and self.grabbed == nil then self.grabbed = {c,self.hand.cards,oldx = c.x, oldy = c.y,ci = #self.hand.cards-i+1,gtime = love.timer.getMicroTime()} break end
				end
				for i = 1,#self.field.cards do
					local c = self.field.cards[#self.field.cards-i+1]
					if c:isOver(xx,yy) and z=='r' then self.zoom = c.image break end
					if c:isOver(xx,yy) and z=='l' and self.grabbed == nil then self.grabbed = {c, self.field.cards, oldx = c.x, oldy = c.y, ci = #self.field.cards-i+1, gtime = love.timer.getMicroTime(),fx = xx,fy = yy} break end
				end
				for i = 1,#self.graveyard.cards do
					local c = self.graveyard.cards[#self.graveyard.cards-i+1]
					if c:isOver(xx,yy) and z=='r' then self.zoom = c.image break end
					if c:isOver(xx,yy) and z=='l' and self.grabbed == nil then self.grabbed = {c, self.graveyard.cards, oldx = c.x, oldy = c.y,ci = #self.graveyard.cards-i+1,gtime = love.timer.getMicroTime()} break end
				end
			else	--exit zoom if you click left or right again
				self.zoom = nil
			end
			
			--draw card from library
			if self:getArea(xx,yy)==self.library then
				self:cardFromTo(#self.library.cards,self.library.cards,self.hand.cards)
				self:adjustHand()
			end
			
			--search library if right click on library
			
			--same for grave
			
			--l or r click to add/lower health, or use mousewheel while over life circle
			if inCircle(xx,yy,(15/800)*sw+self.cardSize*223/2,sw*.18,sw*.05,32) then
				if z=='l' or z=='wu' then self.life = clamp(0,self.life+1,999) end
				if z=='r' or z=='wd' then self.life = clamp(0,self.life-1,999) end
			end
		end
	end
	
	function love.mousereleased(xx,yy,z)
		if self.searching then
			self.libSlider.held = false
			self.graveSlider.held = false
		else
			local g = self.grabbed
			if g then
				if (love.timer.getMicroTime()-g.gtime)<0.2 and g[2]==self.field.cards then		--if grabbed and released on same place, and card is in field, tap it or untap it
					g[1].x = g.oldx
					g[1].y = g.oldy
					g[1].tapped = (g[1].tapped + 1)%2
					self.grabbed = nil
				else		--if grabbed and released different than where grabbed, transfer, adjust(if possible)
					if self:getArea(xx,yy)==nil then
						self:cardFromTo(g.ci,g[2],g[2])
						self.grabbed = nil
					else
						self:cardFromTo(g.ci,g[2],self:getArea(xx,yy).cards)
						self.grabbed = nil
					end
				end
			end
		end
	end
	
end







